import commonFunctions from "../../../custom/property/tools/commonFunctions"


/**
 * 吸附距离
 */
const ADSORBDISTANCE = 5;

/**
 * 提取成组的算法
 * 实现选中多个成组
 * @param {*} layers
 * @param {*} elementObj
 * @param {*} states  状态集合
 * @param {*} selectedState 选中的状态
 */
const createGroup = function (layers, elementObj, states, selectedState) {
    //当源对象为组时才可以拖入
    if (elementObj.elementSource.type.trim() == 'group' && elementObj.elementTarget.id != elementObj.elementSource.id) {
        let group = commonFunctions.getElementByIdRecursion(layers, elementObj.elementSource.id);


        let isRootElement = layers.find(l => l.id == elementObj.elementTarget.id) != undefined;
        if (isRootElement) {
            // 将对象从源集合中删除
            let findState = states.find(s => s.id == selectedState.id);
            if (findState) {
                let findIndex = findState.layers.findIndex(ele => {
                    return ele.id === elementObj.elementTarget.id;
                });
                findState.layers.splice(findIndex, 1);
            }
        } else {
            let parentElement = commonFunctions.getParentElement(elementObj.elementTarget.id, layers);
            // 将对象从源集合中删除
            let findIndex = parentElement.childrens.findIndex(ele => {
                return ele.id === elementObj.elementTarget.id;
            });
            parentElement.childrens.splice(findIndex, 1);
        }

        //默认 在卡片的左上角
        let position = commonFunctions.getPostion(elementObj.elementTarget);
        position.top = 0;
        position.left = 0;

        //如果卡片大小比图表小 则图表按比例缩小
        let sizeChart = commonFunctions.getSize(elementObj.elementTarget);
        let sizeGroup = commonFunctions.getSize(group);

        if (sizeGroup.width < sizeChart.width && sizeGroup.height < sizeChart.height) {
            //计算宽度缩放比例
            let ratioWidth = sizeGroup.width / sizeChart.width;
            //计算高度缩放比例
            let ratioHeight = sizeGroup.height / sizeChart.height;
            let tempRatio = ratioWidth >= ratioHeight ? ratioHeight : ratioWidth;
            sizeChart.height *= tempRatio;
            sizeChart.width *= tempRatio

            commonFunctions.setWidthAndHeightChangePro(elementObj.elementTarget, tempRatio, tempRatio)

        } else if (sizeGroup.width < sizeChart.width) {
            //计算宽度缩放比例
            let ratioWidth = sizeGroup.width / sizeChart.width;
            sizeChart.height *= ratioWidth;
            let widthRadio = sizeGroup.width / sizeChart.width;
            sizeChart.width = sizeGroup.width
            commonFunctions.setWidthAndHeightChangePro(elementObj.elementTarget, ratioWidth, widthRadio)

        } else if (sizeGroup.height < sizeChart.height) {
            //计算高度缩放比例
            let ratioHeight = sizeGroup.height / sizeChart.height;
            sizeChart.width *= ratioHeight;
            let heightRadio = sizeGroup.height / sizeChart.height;
            sizeChart.height = sizeGroup.height
            commonFunctions.setWidthAndHeightChangePro(elementObj.elementTarget, heightRadio, ratioHeight)
        }

        group.childrens.push(elementObj.elementTarget);

        //设置组的Zindex  取最大值
        if (group.zIndex < elementObj.elementTarget.zIndex) {
            group.zIndex = elementObj.elementTarget.zIndex;
        }
    }
}

let isAdsorbY = false;
let offsetYTemp = 0;
/**
 * 计算y贴靠的偏移量
 * @param {行 线集合} columnLineArr
 * @param {*} postion 目标位置
 * @param {*} size  目标大小
 * @param {*} offsetY  y偏移量
 */
const adsorbTopAndBottom = function (rowLineArr, postion, size, offsetY) {
    if (offsetY == 0) return 0;

    if (isAdsorbY) {
        offsetYTemp += offsetY;
        if (Math.abs(offsetYTemp) > ADSORBDISTANCE) {
            isAdsorbY = false;
            return offsetYTemp;
        } else {
            return 0;
        }
    }
    let rowLineArrtem = rowLineArr.map(c => {
        return c.top
    });
    let topCloseNum = findCloseNum(rowLineArrtem, postion.top + offsetY);
    let bottomCloseNum = findCloseNum(rowLineArrtem, postion.top + size.height + offsetY)
    //查找数组中在贴靠距离范围内并且最小距离
    if (topCloseNum == -1 && bottomCloseNum == -1) { //上下都没有贴靠
        return offsetY
    }
    isAdsorbY = true;
    let disanceTop = 10000,
        distanceBottom = 10000;
    if (topCloseNum != -1 && bottomCloseNum != -1) {
        if (disanceTop < distanceBottom || disanceTop == distanceBottom) { //上边贴靠距离小于或等于右边贴靠距离  取上边 否则取下边
            return topCloseNum + 5
        } else {
            return bottomCloseNum + 5;
        }
    } else if (topCloseNum != -1) {
        disanceTop = Math.abs(topCloseNum);
        return topCloseNum + 5;
    } else if (bottomCloseNum != -1) {
        distanceBottom = Math.abs(bottomCloseNum)
        return bottomCloseNum + 5;
    }
}

let isAdsorbX = false;
let offsetXTemp = 0;

/**
 * 计算x贴靠的偏移量
 * @param {列 线集合} columnLineArr
 * @param {*} postion 目标位置
 * @param {*} size  目标大小
 * @param {*} offsetX  x偏移量
 */
const adsorbLeftAndRight = function (columnLineArr, postion, size, offsetX) {
    if (offsetX == 0) return 0;

    if (isAdsorbX) {
        offsetXTemp += offsetX;
        if (Math.abs(offsetXTemp) > ADSORBDISTANCE) {
            isAdsorbX = false;
            return offsetXTemp;
        } else {
            return 0;
        }
    }
    let columnLineArrtem = columnLineArr.map(c => {
        return c.left
    });
    let leftCloseNum = findCloseNum(columnLineArrtem, postion.left + offsetX);
    let rightCloseNum = findCloseNum(columnLineArrtem, postion.left + size.width + offsetX)
    //查找数组中在贴靠距离范围内并且最小距离
    if (leftCloseNum == -1 && rightCloseNum == -1) { //左右都没有贴靠
        return offsetX
    }
    isAdsorbX = true;
    let disanceLeft = 10000,
        distanceRifht = 10000;
    if (leftCloseNum != -1 && rightCloseNum != -1) {
        if (disanceLeft < distanceRifht || disanceLeft == distanceRifht) { //左边贴靠距离小于或等于右边贴靠距离  取左边 否则取右边
            return leftCloseNum + 5
        } else {
            return rightCloseNum + 5;
        }
    } else if (leftCloseNum != -1) {
        disanceLeft = Math.abs(leftCloseNum);
        return leftCloseNum + 5;
    } else if (rightCloseNum != -1) {
        distanceRifht = Math.abs(rightCloseNum)
        return rightCloseNum + 5;
    }
}



/**
 * @method
 * @author  gedesiwen
 * @param {array} arr 需要查找的数组
 * @param {number} num 目标数值，查找的是与这个数值最接近的
 * @return {number} 返回查找到的最接近的数值
 * @desc 获取数组中与目标数值最接近的数值
 */
function findCloseNum(arr, num) {
    var index = 0; // 保存最接近数值在数组中的索引
    var d_value = Number.MAX_VALUE; // 保存差值绝对值，默认为最大数值
    for (var i = 0; i < arr.length; i++) {
        var new_d_value = Math.abs(arr[i] - num); // 新差值
        if (new_d_value <= d_value) { // 如果新差值绝对值小于等于旧差值绝对值，保存新差值绝对值和索引
            if (new_d_value === d_value && arr[i] < arr[index]) { // 如果数组中两个数值跟目标数值差值一样，取大
                continue;
            }
            index = i;
            d_value = new_d_value;
        }
    }

    if (Math.abs(arr[index] - num) < ADSORBDISTANCE) {
        return arr[index] - num;
    } else {
        return -1
    }
}


const modulePage = {
    state: {
        pageInfo: {}, //当前页面的配置信息
        selectedState: null, //当前选中的状态
        selectedArray: [], //选中的元素集合
        isShowLanyerPanel: true, //是否显示图层面板
        isShowChartTabPanel: true, //是否显示组件面板
        isShowPropertyPanel: true, //是否显示属相面板
        canvasWidth: 0, //画板的宽度
        canvasHeight: 0, //画板的高度
        currentSourceId: null, //当前选中的数据源的Id,
        isShowUnLock: false, //是否显示解锁按钮
        isShowUnHidden: false, //是否显示显示按钮
    },
    mutations: {
        //设置页面信息
        setPageInfo(state, pageInfo) {
            state.pageInfo = pageInfo;
            let selectedState = pageInfo.states.find(s => s.isUnfold);
            if (selectedState) {
                this.commit('setSelectedState', JSON.parse(JSON.stringify(selectedState))); //设置当前的选中状态
            }
        },

        //设置当前选中的状态
        setSelectedState(state, selectedState) {
            state.selectedArray = [];
            state.selectedState = selectedState;
        },

        //保存页面信息  暂存在localStorage
        savePageInfo(state) {
            localStorage.setItem("pageInfo", JSON.stringify(state.pageInfo))
        },

        // 更新元素尺寸
        updateElementSize: function (state, payload) {
            // 查找id对应的元素
            var find = commonFunctions.getElementByIdRecursion(commonFunctions.getLayers(state.pageInfo.states, state.selectedState.id), payload.id);
            // 如果找到元素，则更新元素的位置和尺寸
            let selectChatr = state.selectedArray[0];

            let parent = null;

            //边界判断
            if (find != null) {
                //查找当前移动图表的父级元素
                //不能超出父级元素的边界
                let findParentTemp = commonFunctions.getParentElement(selectChatr.id, commonFunctions.getLayers(state.pageInfo.states, state.selectedState.id));
                if (findParentTemp) {
                    if (findParentTemp.id != selectChatr.id) {
                        //找到父级元素
                        parent = findParentTemp;
                    }
                }


                let postion = commonFunctions.getPostion(find);
                let positionSelected = commonFunctions.getPostion(selectChatr);
                let size = commonFunctions.getSize(find);
                let sizeSelected = commonFunctions.getSize(selectChatr);
                //左边界
                if (postion.left < 0) {
                    postion.left = 0;
                }
                let offsetX = 0,
                    offsetY = 0;
                if (payload.isShowGridLine) {
                    offsetX = adsorbLeftAndRight(state.pageInfo.columnLineArr, postion, size, payload.offsetX);
                    if (isAdsorbX && offsetX != 0) {
                        offsetXTemp = 0;
                    }
                    offsetY = adsorbTopAndBottom(state.pageInfo.rowLineArr, postion, size, payload.offsetY);
                    if (isAdsorbY && offsetY != 0) {
                        offsetYTemp = 0;
                    }
                } else {
                    offsetX = payload.offsetX;
                    offsetY = payload.offsetY;
                }

                //右边界 --不限定
                if (offsetX > 0) {
                    // if (postion.left + size.width + 10 <= state.canvasWidth) {
                    //     postion.left += offsetX;
                    //     positionSelected.left += offsetX;
                    // }
                    if (parent) {
                        let parentSize = commonFunctions.getSize(parent);
                        if (postion.left + size.width <= parentSize.width) {
                            postion.left += offsetX;
                            positionSelected.left += offsetX;
                        }
                    } else {
                        postion.left += offsetX;
                        positionSelected.left += offsetX;
                    }

                } else {
                    postion.left += offsetX;
                    positionSelected.left += offsetX;
                }

                //上边界
                if (postion.top < 0) {
                    postion.top = 0;
                    positionSelected.top = 0;
                }

                //下边界  --不限定
                if (offsetY > 0) {
                    // if (size.height + postion.top + 10 <= state.canvasHeight) {
                    //     postion.top += offsetY;
                    //     positionSelected.top += offsetY;
                    // }
                    if (parent) {
                        let parentSize = commonFunctions.getSize(parent);
                        if (postion.top + size.height <= parentSize.height) {
                            postion.top += offsetY;
                            positionSelected.top += offsetY;
                        }
                    } else {
                        postion.top += offsetY;
                        positionSelected.top += offsetY;
                    }

                } else {
                    postion.top += offsetY;
                    positionSelected.top += offsetY;
                }


                //记录变化的高宽比例值
                let heightRadio = payload.offsetHeight / size.height;
                let widthRadio = payload.offsetWidth / size.width;

                size.width += payload.offsetWidth;
                size.height += payload.offsetHeight;

                sizeSelected.width += payload.offsetWidth;
                sizeSelected.height += payload.offsetHeight;

                //如果选中项是组 则递归改变子组件的大小
                if (find.type == 'group') {
                    size.defaultWidth = sizeSelected.defaultWidth = size.width;
                    size.defaultHeight = sizeSelected.defaultHeight = size.height;
                    find.childrens.forEach(ele => {
                        commonFunctions.setWidthAndHeight(ele, heightRadio, widthRadio)
                    });
                }
                // commonFunctions.changeGroupPosition(payload.id, commonFunctions.getLayers(state.pageInfo.states, state.selectedState.id))
            }
        },

        // 向选中集合中追加对象
        addSelectedElement: function (state, payload) {
            state.selectedArray.push(JSON.parse(JSON.stringify(payload.element)));
        },

        //将元素添加到当前页面集合中
        addElement(state, selectChar) {
            selectChar.chart.id = commonFunctions.getGuid();
            //增加图表尺寸和图表位置样式

            selectChar.chart.style.push({
                "category1": "基础属性",
                "category2": "",
                "display": "图表位置",
                "name": "postion",
                "top": 0,
                "left": 0,
                "type": "NumberTwo",
                "isAdvanced": false,
                "isShortcut": false,
                "isHidden": false
            });

            //默认属性设置
            selectChar.chart.isLock = false;
            selectChar.chart.isHidden = false;
            selectChar.chart.zIndex = 0;
            selectChar.chart.draggable = true;

            //初始化默认属性
            let top = 0,
                left = 0,
                centerX = state.canvasWidth / 2,
                centerY = state.canvasHeight / 2,
                size = commonFunctions.getSize(selectChar.chart);


            if (!selectChar.top || selectChar.top == 0) {
                top = centerY - (size.height / 2);
            } else {
                top = selectChar.top - (size.height / 2);
            }
            if (!selectChar.left || selectChar.left == 0) {
                left = centerX - (size.width / 2);
            } else {
                left = selectChar.left - (size.width / 2);
            }
            //边界判定
            if (top < 0) {
                top = 0;
            }
            if (top + size.height > state.canvasHeight) {
                top = state.canvasHeight - size.height;
            }

            if (left < 0) {
                left = 0;
            }
            if (left + size.width > state.canvasWidth) {
                left = state.canvasWidth - size.width;
            }

            let postion = commonFunctions.getPostion(selectChar.chart);
            postion.top = top;
            postion.left = left;
            let findState = state.pageInfo.states.find(s => s.id == state.selectedState.id);
            findState.layers.push(selectChar.chart);
        },

        // 从选中集合中删除对象
        removeSelectedElement: function (state, payload) {
            let findIndex = state.selectedArray.findIndex(a => {
                return a.id === payload.element.id
            });

            state.selectedArray.splice(findIndex, 1);
        },
        // 清空选中集合
        clearSelectedArray: function (state) {
            state.selectedArray.splice(0);
        },

        // 计算对象是否被选中
        isElementSelected: function (state, payload) {
            var isSelected = false;
            var array = state.selectedArray;
            for (var i = 0, length = array.length; i < length; i++) {
                var it = array[i];
                if (it.id === payload.element.id) {
                    isSelected = true;
                    break;
                }
            }
            return isSelected;
        },

        /**
         * 拆组
         * 为保证位置不变 拆组时需要给子元素加上top和left
         * 拆组 只执行一级拆组不会递归
         * 可多个组同时拆
         * @param {*} state
         */
        unGrouping: function (state) {
            state.selectedArray.forEach(select => {
                if (select.type == "group") {
                    //元素重定位
                    commonFunctions.setTopAndLeft(select, false);

                    //找到父级元素
                    let parent = commonFunctions.getParentElement(select.id, state.pageInfo.layers)

                    //判断是否为根元素
                    let isRootElement = state.pageInfo.layers.find(l => l.id == select.id)

                    if (isRootElement) {
                        select.childrens.forEach(children => {
                            //添加源集合中
                            children.draggable = true;
                            state.pageInfo.layers.push(children);
                        });
                        //在源数据中删除当前组
                        let findEleIndex = state.pageInfo.layers.findIndex(ele => ele.id === select.id);
                        state.pageInfo.layers.splice(findEleIndex, 1);
                    } else {
                        select.childrens.forEach(children => {
                            //添加源集合中
                            parent.childrens.push(children);
                        });
                        //在源数据中删除当前组
                        let findEleIndex = parent.childrens.findIndex(ele => ele.id === select.id);
                        parent.childrens.splice(findEleIndex, 1);
                    }
                }
            });
            //清空选中集合
            state.selectedArray.splice(0);
        },

        /**
         * 成组
         * 如果有选中的选中的加入组中，
         * 如果没有选中的 新建一个控组
         * @param {*} state
         */
        grouping: function (state) {
            let group = {
                "id": commonFunctions.getGuid(),
                "name": `组${Date.parse(new Date())}`,
                "type": "group",
                "style": [{
                    "name": "size",
                    "display": "图表尺寸",
                    "width": 0,
                    "height": 0,
                    "isRelevance": false,
                    "type": "NumberRelevance",
                    "category1": "基础属性",
                    "category2": "图表尺寸",
                    "isAdvanced": false,
                    "isShortcut": false,
                    "isHidden": false
                },
                {
                    "name": "postion",
                    "display": "图表位置",
                    "top": 0,
                    "left": 0,
                    "type": "NumberTwo",
                    "category1": "基础属性",
                    "category2": "图表位置",
                    "isAdvanced": false,
                    "isShortcut": false,
                    "isHidden": false
                },
                {
                    "name": "opacity",
                    "display": "透明度",
                    "type": "slider",
                    "default": 1,
                    "minValue": 0,
                    "maxValue": 1,
                    "precision": 0,
                    "step": 0.1,
                    "value": 1,
                    "category1": "基础属性",
                    "category2": "透明度",
                    "isAdvanced": false,
                    "isShortcut": false,
                    "isHidden": false
                }
                ],
                //缩略图
                "thumbnailUrl": "",
                //图表类
                "ico": "",
                //是否拖拽
                "isLock": false,
                //是否隐藏
                "isHidden": false,
                //层级索引
                "zIndex": 0,
                //是否可拖拽
                "draggable": true,
                "component": {},
                "childrens": []
            };

            if (state.selectedArray.length <= 0) {
                state.pageInfo.layers.push(group);
                return;
            }

            let isRootElement = state.pageInfo.layers.find(l => l.id == state.selectedArray[0].id) != undefined;

            if (isRootElement) {
                // 将组添加到源集合中
                state.pageInfo.layers.push(group);
                state.selectedArray.forEach(s => {
                    createGroup(state.pageInfo.layers, {
                        elementSource: group,
                        elementTarget: s
                    });
                });
            } else {
                //找到当前选中项的父元素
                let parent = commonFunctions.getParentElement(state.selectedArray[0].id, state.pageInfo.layers)

                if (parent.type == 'group') {
                    parent.childrens.push(group);
                    state.selectedArray.forEach(s => {
                        createGroup(parent.childrens, {
                            elementSource: group,
                            elementTarget: s
                        });
                    });
                } else {
                    // 将组添加到源集合中
                    state.pageInfo.layers.push(group);
                    state.selectedArray.forEach(s => {
                        createGroup(state.pageInfo.layers, {
                            elementSource: group,
                            elementTarget: s
                        });
                    });
                }
            }
        },

        /**
         * 添加一个对象到当前组内
         * 为保证位置不变 添加时需要给子元素减去top和left
         * @param {*} state
         * @param {*} elementObj  {elementSource源目标    elementTarget将要添加的对象}
         */
        addElementToGroup: function (state, elementObj) {
            createGroup(commonFunctions.getLayers(state.pageInfo.states, state.selectedState.id), elementObj, state.pageInfo.states, state.selectedState);
            //清空选中集合
            state.selectedArray.splice(0);
        },

        /**
         * 添加一个事件触发与处理函数的关联到当前选中的元素，添加成功返回 true
         * @param {*} state
         * @param {*} handlerObj  {将要添加的事件处理函数详情}
         */
        addEventHandler: function (state, handlerObj, success, fail) {
            const STRING_PARAM_INVALID = '添加事件处理时参数无效。';
            const STRING_DEF_NOT_FOUND = '未找到名称为“{0}”的事件定义。';
            const STRING_EXISTED = '已经存在指向“{0}”组件 ({1}) 的“{2}”函数的事件处理。';
            ///////////////////////////////////////
            // params validation start
            ///////////////////////////////////////
            // check incoming handler is valid
            if (!handlerObj || !handlerObj.eventName || !handlerObj.targetId || !handlerObj.targetName || !handlerObj.functionName) {
                window.console.error(STRING_PARAM_INVALID);
                if (fail) {
                    fail(STRING_PARAM_INVALID);
                }
                return false;
            }

            // check [chartname].json has at least one event definition
            if (!state.selectedArray[0].component.events) {
                window.console.error(STRING_DEF_NOT_FOUND.replace('{0}', handlerObj.name));
                if (fail) {
                    fail(STRING_DEF_NOT_FOUND.replace('{0}', handlerObj.name));
                }
                return false;
            }

            // check incoming handler(event) match one of pre-defined events in [chartname].json
            for (let _event of state.selectedArray[0].component.events) {
                if (_event.name && (_event.name === handlerObj.eventName)) {
                    // event definition match, look for existing handler (target id & functionName match)
                    for (let _handler of _event.handlers) {
                        if ((_handler.targetId && (_handler.targetId === handlerObj.targetId)) && (_handler.functionName && (_handler.functionName === handlerObj.functionName))) {
                            // handler already exists, warn and return
                            window.console.warn(STRING_EXISTED.replace('{0}', handlerObj.targetName).replace('{1}', handlerObj.targetId).replace('{2}', handlerObj.functionName));
                            if (fail) {
                                fail(STRING_EXISTED.replace('{0}', handlerObj.targetName).replace('{1}', handlerObj.targetId).replace('{2}', handlerObj.functionName));
                            }
                            return false;
                        }
                    }
                    // proceed to add handler
                    _event.handlers.push({
                        targetId: handlerObj.targetId,
                        targetName: handlerObj.targetName,
                        functionName: handlerObj.functionName
                    });

                    window.console.log('chart is ready.');
                    window.console.log(state.selectedArray[0]);
                    window.console.log(JSON.stringify(state.selectedArray[0]));


                    window.console.log('about to compare layer', state.pageInfo.layers);
                    // look for chart instance node in pageInfo
                    for (let _layer of state.pageInfo.layers) {
                        window.console.log('comparing layer', _layer.id, state.selectedArray[0].id);
                        if (_layer.id === state.selectedArray[0].id) {
                            window.console.log('found layer', _layer.id);
                            _layer.component.events = state.selectedArray[0].component.events;
                            break;
                        }
                    }

                    if (success) {
                        success();
                    }
                    return true;
                }
            }
            // no event definition match
            window.console.error(STRING_DEF_NOT_FOUND.replace('{0}', handlerObj.name));
            if (fail) {
                fail(STRING_DEF_NOT_FOUND.replace('{0}', handlerObj.name));
            }
            return false;
        }
    }
};

export default modulePage;