// 常用函数

// 计算元素是否被选中
const isElementSelected = function(selectedArray, element) {
  let selected = selectedArray.find((s) => s.id === element.id);
  return selected != undefined;
};
// 获取时间的格式化字符串
let timeFormat = function(date, formatString = 'hh:mm:ss') {
  var padLeftZero = function(str) {
    return ('00' + str).substr(str.length);
  };

  if (/(y+)/.test(formatString)) {
    formatString = formatString.replace(
      RegExp.$1,
      (date.getFullYear() + '').substr(4 - RegExp.$1.length)
    );
  }
  let o = {
    'M+': date.getMonth() + 1,
    'd+': date.getDate(),
    'h+': date.getHours(),
    'm+': date.getMinutes(),
    's+': date.getSeconds(),
  };
  for (let k in o) {
    if (new RegExp(`(${k})`).test(formatString)) {
      let str = o[k] + '';
      formatString = formatString.replace(
        RegExp.$1,
        RegExp.$1.length === 1 ? str : padLeftZero(str)
      );
    }
  }
  return formatString;
};
/**
 * 递归设置是否可拖拽属性为不可拖拽
 * @param {*} element
 */
const setUnDraggable = function(element) {
  if (element.children.length != 0) {
    element.children.forEach((ele) => {
      ele.draggable = false;
      setUnDraggable(ele);
    });
  }
  element.draggable = false;
};

/**
 * 设置属性
 * @param {*} element
 * @param {*属性名称} propertyName
 * @param {*属性集合} propertyValue
 */
const setProperty = function(element, propertyName, propertyValue) {
  if (element.childrens.length != 0) {
    element.childrens.forEach((ele) => {
      if (ele[propertyName] != undefined) {
        ele[propertyName] = propertyValue;
      }
      setProperty(ele, propertyName, propertyValue);
    });
  }
  if (element[propertyName] != undefined) element[propertyName] = propertyValue;
};

/**
 * 生成唯一标识
 */
const getGuid = function() {
  let time = new Date().getTime();
  if (window.performance && typeof window.performance.now === 'function') {
    time += performance.now();
  }
  let guid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(
    c
  ) {
    let r = (time + Math.random() * 16) % 16 | 0;
    time = Math.floor(time / 16);
    return (c == 'x' ? r : (r & 0x3) | 0x8).toString(16);
  });
  return guid;
};

/**
 * 获取图表尺寸
 * @param {layer} element
 */
const getSize = function(element) {
  if (!element.style) return null;
  return element.style.find((s) => s.display == '图表尺寸');
};

/**
 * 获取图表的位置
 * @param {layer} element
 */
const getPostion = function(element) {
  if (!element.style) return null;
  return element.style.find((s) => s.display == '图表位置');
};

/**
 * 获取随机颜色的方法
 * @param alpha 透明度 0-1
 * @returns {string}
 */
const getRandomRGBAColor = function(alpha) {
  let r = Math.round(Math.random() * 255);
  let g = Math.round(Math.random() * 255);
  let b = Math.round(Math.random() * 255);
  if (alpha === undefined) {
    return 'rgba(' + r + ',' + g + ',' + b + ',' + 1 + ')';
  } else {
    return 'rgba(' + r + ',' + g + ',' + b + ',' + alpha + ')';
  }
};

const colors = [
  '#FFFF00FF',
  '#BDFD00FF',
  '#72D803FF',
  '#00A880FF',
  '#009ACFFF',
  '#9673FFFF',
  '#DC73FFFF',
  '#FF7A4DFF',
  '#FF9326FF',
  '#FFC926FF',
];

/**
 * 获取随机的16进制颜色
 */
const getRandom16Color = function(index) {
  // let r = Math.round(Math.random() * 255);
  // let g = Math.round(Math.random() * 255);
  // let b = Math.round(Math.random() * 255);
  // return '#' + r.toString(16) + g.toString(16) + b.toString(16);
  return colors[index % 10];
};

/**
 * 判断是否包含标记
 * @param {字符} stringCode
 */
const isContainMark = function(stringCode) {
  let isContain;
  if (stringCode) {
    stringCode = stringCode.trim();
    isContain =
      stringCode.indexOf('{') == 0 &&
      stringCode.indexOf('}') == stringCode.length - 1;
  }
  return isContain;
};

//添加千分位
const thousandthPercentile = function(s, n) {
  n = n >= 0 && n <= 20 ? n : 2;
  s = parseFloat((s + '').replace(/[^\d\.-]/g, '')).toFixed(n) + '';
  let l = s
    .split('.')[0]
    .split('')
    .reverse();
  var t = '';
  for (var i = 0; i < l.length; i++) {
    t += l[i] + ((i + 1) % 3 == 0 && i + 1 != l.length ? ',' : '');
  }

  let result = t
    .split('')
    .reverse()
    .join('');

  if (s.split('.').length > 1) {
    result = result + '.' + s.split('.')[1];
  }

  return result;
};

//判断数组中是否包含值
/**
 * 判断数组中是否包含值
 * @param {Object} parm - 判断类型
 * @param {String} parm.type - 数组类型
 * @param {Object} parm.value - 判断的值
 * @param {String} parm.name - 对象类型的属性名
 */
const contain = function(parm) {
  let index = -1;
  if (!parm.type || parm.type == 'string') {
    index = parm.arr.indexOf((s) => s == parm.value);
  }
  if (parm.type == 'object') {
    index = parm.arr.indexOf((s) => s[parm.name] == parm.value);
  }
  return index > -1;
};

/* 获取场景地图的风格字符串 */
const getSceneMapPattern = function(pattern) {
  let result = 'default';
  if (pattern == '青色' || pattern == '青色风格') {
    result = 'cyan';
  } else if (pattern == '蓝色' || pattern == '蓝色风格') {
    result = 'blue';
  }

  return result;
};

/* 随机字符串 */
const randomString = function(len) {
  len = len || 32;
  var $chars =
    'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz2345678'; /****默认去掉了容易混淆的字符oOLl,9gq,Vv,Uu,I1****/
  var maxPos = $chars.length;
  var pwd = '';
  for (let i = 0; i < len; i++) {
    pwd += $chars.charAt(Math.floor(Math.random() * maxPos));
  }
  return pwd;
};

/* 折现图颜色渐变 */
const discountColor = function(color, transparent) {
  let val = null;
  if (color.indexOf('#') > -1) {
    if (color.length > 8) {
      let alpha = color.slice(7, 9);
      alpha = +(parseInt(alpha, 16) / 255).toFixed(2);
      val =
        'rgba(' +
        parseInt('0x' + color.slice(1, 3)) +
        ',' +
        parseInt('0x' + color.slice(3, 5)) +
        ',' +
        parseInt('0x' + color.slice(5, 7)) +
        ',' +
        transparent +
        ')';
    } else {
      val =
        'rgba(' +
        parseInt('0x' + color.slice(1, 3)) +
        ',' +
        parseInt('0x' + color.slice(3, 5)) +
        ',' +
        parseInt('0x' + color.slice(5, 7)) +
        ',' +
        transparent +
        ')';
    }
  }
  return val;
};

export default {
  timeFormat,
  isElementSelected,
  setUnDraggable,
  setProperty,
  getGuid,
  getSize,
  getPostion,
  getRandomRGBAColor,
  getRandom16Color,
  isContainMark,
  thousandthPercentile,
  contain,
  getSceneMapPattern,
  randomString,
  discountColor,
};
