import chartUtils from '../../utils/chart-utils.js';

/*--------------------------------外部调用----------------------------*/

/* 所有的地标图层 */
const currentLayer = [];

/* 封装 - 在容器中添加图层 */
const addLayer = function (container, layerInfo) {
  logger.debug(`图表调试 - __component_chinese__ - addLayer`, container, layerInfo);

  //先删除图层
  for (let index = currentLayer.length - 1; index >= 0; index--) {
    if (currentLayer[index].name === layerInfo.id) {
      currentLayer.splice(index, 1);
    }
  }
  let layer = getLayer(layerInfo);
  updateToContainer(container, layerInfo, layer);
};

/* 封装 - 在容器中移除图层 */
const removeLayer = function (container, layerInfo) {
  logger.debug(`图表调试 - __component_chinese__ - removeLayer`, container, layerInfo);

  let layer = getLayer(layerInfo);
  deleteLayer(container, layer);
};

/* 封装 - 在容器中显隐改变 */
const visibleChange = function (container, layerInfo) {
  logger.debug(`图表调试 - __component_chinese__ - visibleChange`, container, layerInfo);
  layerVisible(container, layerInfo);
};

/* 封装 - 在容器中样式改变 */
const styleChange = function (container, layerInfo, newStyle) {
  logger.debug(`图表调试 - __component_chinese__ - styleChange`, container, layerInfo, newStyle);

  let styles = layerInfo.component.style;
  let oldStyle = styles.find((s) => s.name.toLowerCase() == newStyle.name.toLowerCase());
  if (oldStyle) {
    oldStyle.value = newStyle.value;

    let layer = getLayer(layerInfo);
    layer.style = getStyle(layerinfo);
    updateLayer(container, layerInfo, layer);
  }
};

/* 封装 - 在容器中数据改变 */
const dataChange = function (container, layerInfo, newData) {
  logger.debug(`图表调试 - __component_chinese__ - dataChange`, container, layerInfo, newData);

  let layer = getLayer(layerInfo);
  layer.fields = getDataSetting(layerInfo).fields;
  layer.rowData = newData;
  updateLayer(container, layerInfo, layer);
};

/*--------------------------------创建图层----------------------------*/

/* 获取图层 */
const getLayer = function (layerInfo) {
  let layer = currentLayer.find((l) => l.name == layerInfo.id);
  if (layer) {
    return layer;
  }

  layer = {
    name: layerInfo.id,
  };

  let dataSetting = getDataSetting(layerInfo);
  layer.fields = dataSetting.fields;
  layer.rowData = dataSetting.rowData;
  layer.points = getPoints(layer);
  layer.style = getStyle(layerInfo);
  currentLayer.push(layer);

  return layer;
};

/* 更新到容器 */
const updateToContainer = function (container, layerInfo, layer) {
  for (let point of layer.points) {
    let canvas = createCanvas(layerInfo, point.label);
    addPoint(container, layer.name, point, layer.style, canvas);
  }
};

/*--------------------------------删除图层----------------------------*/

/* 删除图层 */
const deleteLayer = function (container, layer) {
  for (let point of layer.points) {
    container.removeLandmark(layer.name, point.guid);
  }
  layer.points = [];
};

/*--------------------------------更新图层----------------------------*/

/* 更新图层 */
const updateLayer = function (container, layerinfo, layer) {
  deleteLayer(container, layer);
  layer.points = getPoints(layer);
  updateToContainer(container, layerinfo, layer);
};

/* 图层显隐 */
const layerVisible = function (container, layerInfo) {
  let layer = getLayer(layerInfo);
  if (layerInfo.show) {
    layer.points = getPoints(layer);
    updateToContainer(container, layerInfo, layer);
  } else {
    deleteLayer(container, layer);
  }
};

/*--------------------------------其他----------------------------*/

/* 获取点 */
const getPoints = function (layer) {
  let fields = layer.fields;
  let rowData = layer.rowData;
  if (!fields || !rowData) {
    return [];
  }

  let lonName = chartUtils.getFieldBindingName(fields, 'lon');
  let latName = chartUtils.getFieldBindingName(fields, 'lat');

  let altName = chartUtils.getFieldBindingName(fields, 'alt');
  let batchName = chartUtils.getFieldBindingName(fields, 'batch');
  let categoryName = chartUtils.getFieldBindingName(fields, 'category');
  let labelName = chartUtils.getFieldBindingName(fields, 'label');

  var points = [];
  for (let row of rowData) {
    let point = {
      lon: 0,
      lat: 0,
      alt: 0,
      batch: '',
      category: '',
      label: '',
      guid: chartUtils.getGuid(),
    };

    let lon = row[lonName];
    let lat = row[latName];

    if (typeof lon !== 'number' || typeof lat !== 'number') {
      continue;
    } else {
      point.lon = lon;
      point.lat = lat;
    }

    if (altName) {
      let alt = row[altName];
      if (typeof alt === 'number') {
        point.alt = alt;
      }
    }

    if (batchName) {
      let batch = row[batchName];
      if (batch) {
        point.batch = batch;
      }
    }

    if (categoryName) {
      let category = row[categoryName];
      if (category) {
        point.category = category;
      }
    }

    if (labelName) {
      let label = row[labelName];
      if (label) {
        point.label = label;
      }
    }

    points.push(point);
  }

  return points;
};

/* 获取样式 */
const getStyle = function (layerInfo) {
  let styles = layerInfo.component.style;
  //图标
  let icon = chartUtils.getStyleValue(styles, 'globalStyleIcon');
  let brightness = chartUtils.getStyleValue(styles, 'globalStyleBrightness');
  //图标缩放
  let enableScaling = chartUtils.getStyleValue(styles, 'globalStyleDistanceScaling');
  let smallIconMax = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingSmallMax');
  let smallIconMin = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingSmallMin');
  let bigIconMax = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingBiglMax');
  let bigIconMin = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingBigMin');
  let smallIconFactor = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingSmallFactor');
  let bigIconFactor = chartUtils.getStyleValue(styles, 'globalStyleDistanceScalingBigFactor');
  //图标偏移
  let enableOffset = chartUtils.getStyleValue(styles, 'offsetSetting');
  let lonOffset = chartUtils.getStyleValue(styles, 'offsetSettingLonOffset');
  let latOffset = chartUtils.getStyleValue(styles, 'offsetSettingLatOffset');
  let altOffset = chartUtils.getStyleValue(styles, 'offsetSettingAltOffset');
  //标签偏移
  let enableLabel = chartUtils.getStyleValue(styles, 'label');
  let labelHorOffset = chartUtils.getStyleValue(styles, 'labelHorOffset');
  let labelVerOffset = chartUtils.getStyleValue(styles, 'labelVerOffset');
  //启用贴地
  let staickLand = chartUtils.getStyleValue(styles, 'offsetSettingEnableStickLand');

  return {
    icon: icon,
    brightness: brightness,
    enableScaling: enableScaling,
    smallIconMax: smallIconMax,
    smallIconMin: smallIconMin,
    bigIconMax: bigIconMax,
    bigIconMin: bigIconMin,
    smallIconFactor: smallIconFactor,
    bigIconFactor: bigIconFactor,
    enableOffset: enableOffset,
    lonOffset: lonOffset,
    latOffset: latOffset,
    altOffset: altOffset,
    enableLabel: enableLabel,
    labelHorOffset: labelHorOffset,
    labelVerOffset: labelVerOffset,
    staickLand: staickLand,
  };
};

/* 添加地标点 */
const addPoint = function (container, layerName, point, style, canvas) {
  var pos = new avw.Position('Default');
  pos = pos.set(point.lon, point.alt, point.lat);

  var landMarkInfo = {
    type: 'landmark',
    name: point.guid,
    position: pos,
    icon: style.icon,
    width: 10,
    height: 10,
    iconXOffset: 0,
    iconYOffset: 0,

    scale: style.enableScaling,
    smallIconMax: style.smallIconMax,
    smallIconMin: style.smallIconMin,
    bigIconMax: style.bigIconMax,
    bigIconMin: style.bigIconMin,
    smallIconScale: style.smallIconFactor,
    bigIconScale: style.bigIconFactor,

    label: style.enableLabel,
    labelXOffset: style.labelHorOffset,
    labelYOffset: style.labelVerOffset,
    labelFontSize: canvas.fontSize,
    labelCanvas: canvas.canvas,

    offset: style.enableOffset,
    lonOffset: style.lonOffset,
    latOffset: style.latOffset,
    altOffset: style.altOffset,

    enableHeightMap: style.staickLand,
  };
  container.addLandmark(layerName, landMarkInfo);
};

/* 获取标签 */
const createCanvas = function (layerInfo, labelText) {
  let styles = layerInfo.component.style;
  let textSty = chartUtils.getStyleValue(styles, 'labelTextStyle');
  let styObj = chartUtils.getTextStyleObj(textSty.value[0].items);

  //创建canvas部分
  let _canvas = document.createElement('canvas');
  let _ctx = _canvas.getContext('2d');

  _ctx.font = `${styObj.fontStyle} ${styObj.fontWeight} ${styObj.fontSize}px ${styObj.fontFamily}`;
  _ctx.textAlign = 'center';
  _ctx.textBaseline = 'middle';

  _canvas.width = (_ctx.measureText(labelText).width + styObj.fontSize) * 2;
  _canvas.height = styObj.fontSize * 3.6;

  _ctx.fillStyle = styObj.backgroundColor;
  _ctx.fillRect(0, 0, _canvas.width, _canvas.height);

  _ctx.font = `${styObj.fontStyle} ${styObj.fontWeight} ${styObj.fontSize * 2}px ${styObj.fontFamily}`;
  _ctx.fillStyle = styObj.color;
  _ctx.textAlign = 'center';
  _ctx.textBaseline = 'middle';
  _ctx.fillText(labelText, _canvas.width / 2, _canvas.height / 2);

  return {
    canvas: _canvas,
    fontSize: styObj.fontSize,
  };
};

/* 添加标牌 */
const addPanel = function (container, layerInfo, point) {
  let styles = layerInfo.component.style;

  let bgColor = chartUtils.getStyleValue(styles, 'tooltipBackgroundColor');
  let bdColor = chartUtils.getStyleValue(styles, 'tooltipBorderColor');
  let isShowShadow = chartUtils.getStyleValue(styles, 'tooltipIsShowShadow');
  let textSty = chartUtils.getStyleValue(styles, 'tooltipTextStyle');
  let styObj = chartUtils.getTextStyleObj(textSty.value[0].items);

  let fontStyle = `style="font-size: ${styObj.fontSize}; font-style: ${styObj.fontStyle}; color: ${styObj.color}; background-color: ${styObj.backgroundColor}; font-family: ${styObj.fontFamily}; font-weight: ${styObj.fontWeight};  margin: 5px;"`;

  let content = `<div style="background-color: ${bgColor.color}; border: 1px solid ${bdColor.color}; text-align: left;">`;
  content += `<span ${fontStyle}>经度：${point.lon}</span><br>`;
  content += `<span ${fontStyle}>维度${point.lat}</span><br>`;
  content += `<span ${fontStyle}>高度：${point.alt}</span><br>`;
  content += `<span ${fontStyle}>批号：${point.batch}</span><br>`;
  content += `<span ${fontStyle}>类别：${point.category}</span><br>`;
  content += `<span ${fontStyle}>标签：${point.label}</span></div>`;

  let horOffset = chartUtils.getStyleValue(styles, 'tooltipHorOffset');
  let verOffset = chartUtils.getStyleValue(styles, 'tooltipVerOffset');
  container.addInfoPanel({
    name: point.batch,
    type: 'infopanel',
    exclusive: false,
    content: content,
    position: new avw.Position('Default').set(0, 0, 0),
    offset: [horOffset, verOffset, 0],
    width: 200,
    height: 'auto',
  });
};

/* 获取数据设置 */
const getDataSetting = function (layerInfo) {
  let category = layerInfo.component.data.category;

  switch (category) {
    case '静态输入':
      return layerInfo.component.data.static;
    case '数据接口':
      return layerInfo.component.data.interface;
    case '数据源':
      return layerInfo.component.data.datasource;
  }

  return null;
};

export default {
  addLayer,
  removeLayer,
  visibleChange,
  styleChange,
  dataChange,
};
